@@ -1,15 +1,7 @@
-# This must be done BEFORE importing numpy or anything else.
-# Therefore it must be in your main script.
 import os
-
-os.environ["MKL_NUM_THREADS"] = "1"
-os.environ["NUMEXPR_NUM_THREADS"] = "1"
-os.environ["OMP_NUM_THREADS"] = "1"
-
 import numpy as np
 import pyqmc.mc as mc
 import sys
-import pandas as pd
 import h5py
 
 
@@ -61,7 +53,6 @@ def dmc_propagate(
     accumulators=None,
     ekey=("energy", "total"),
     drift_limiter=limdrift,
-    stepoffset=0,
 ):
     """
     Propagate DMC without branching
@@ -83,7 +74,6 @@ def dmc_propagate(
 
       drift_limiter: a function that takes a gradient and a cutoff and returns an adjusted gradient
 
-      stepoffset: what to start the step numbering at.
 
     Returns: (df,coords,weights)
       df: A list of dictionaries nstep long that contains all results from the accumulators.
@@ -98,10 +88,8 @@ def dmc_propagate(
     wf.recompute(configs)
 
     eloc = accumulators[ekey[0]](configs, wf)[ekey[1]].real
-    # eref_mean = np.mean(weights * eloc) / np.mean(weights)
-    # eref = eref_mean
     df = []
-    for step in range(nsteps):
+    for _ in range(nsteps):
         acc = np.zeros(nelec)
         for e in range(nelec):
             # Propose move
@@ -144,23 +132,49 @@ def dmc_propagate(
 
         avg = {}
         for k, accumulator in accumulators.items():
-            if k != ekey[0]:
-                dat = accumulator(configs, wf)
-            else:
-                dat = energydat
+            dat = accumulator(configs, wf) if k != ekey[0] else energydat
             for m, res in dat.items():
                 avg[k + m] = np.einsum("...i,i...->...", weights, res) / (
                     nconfig * wavg
                 )
         avg["weight"] = wavg
-        avg["weightvar"] = np.std(weights)
-        avg["weightmin"] = np.amin(weights)
-        avg["weightmax"] = np.amax(weights)
         avg["acceptance"] = np.mean(acc)
-        avg["step"] = stepoffset + step
-
         df.append(avg)
-    return pd.DataFrame(df), configs, weights
+    weight = np.asarray([d["weight"] for d in df])
+    avg_weight = weight / np.mean(weight)
+    df_ret = {
+        k: np.mean([d[k] * w for d, w in zip(df, avg_weight)], axis=0)
+        for k in df[0].keys()
+    }
+
+    df_ret["weight"] = np.mean(weight)
+
+    return df_ret, configs, weights
+
+
+def dmc_propagate_parallel(wf, configs, weights, client, npartitions, *args, **kwargs):
+    config = configs.split(npartitions)
+    weight = np.split(weights, npartitions)
+    runs = [
+        client.submit(dmc_propagate, wf, conf, wt, *args, **kwargs)
+        for conf, wt in zip(config, weight)
+    ]
+    allresults = list(zip(*[r.result() for r in runs]))
+    configs.join(allresults[1])
+    weights = np.concatenate(allresults[2])
+    confweight = np.array([len(c.configs) for c in config], dtype=float)
+    confweight_avg = confweight / (np.mean(confweight) * npartitions)
+    weight = np.array([w["weight"] for w in allresults[0]])
+    weight_avg = weight / np.mean(weight)
+    block_avg = {k: np.sum(
+            [
+                res[k] * ww * cw
+                for res, cw, ww in zip(allresults[0], confweight_avg, weight_avg)
+            ],
+            axis=0,
+        ) for k in allresults[0][0].keys()}
+    block_avg["weight"] = np.mean(weight)
+    return block_avg, configs, weights
 
 
 def limit_timestep(weights, elocnew, elocold, eref, start, stop):
@@ -197,13 +211,12 @@ def limit_timestep(weights, elocnew, elocold, eref, start, stop):
     )
     eloc = np.stack([elocnew, elocold])
     fbet = np.amax(eref - eloc, axis=0)
-    tdamp = np.clip((1 - (fbet - start)) / (stop - start), 0, 1)
-    return tdamp
+    return np.clip((1 - (fbet - start)) / (stop - start), 0, 1)
 
 
 def branch(configs, weights):
     """
-    Perform branching on a set of walkers  by stochastic reconfiguration
+    Perform branching on a set of walkers using the 'stochastic comb'
 
     Walkers are resampled with probability proportional to the weights, and the new weights are all set to be equal to the average weight.
     
@@ -217,11 +230,12 @@ def branch(configs, weights):
 
       weights: (nconfig,) all weights are equal to average weight
     """
+
     nconfig = configs.configs.shape[0]
-    wtot = np.sum(weights)
-    probability = np.cumsum(weights / wtot)
+    probability = np.cumsum(weights)
+    wtot = probability[-1]
     base = np.random.rand()
-    newinds = np.searchsorted(probability, (base + np.arange(nconfig) / nconfig) % 1.0)
+    newinds = np.searchsorted(probability, (base + np.linspace(0,wtot,nconfig)) % wtot)
     configs.resample(newinds)
     weights.fill(wtot / nconfig)
     return configs, weights
@@ -233,12 +247,11 @@ def dmc_file(hdf_file, data, attr, configs, weights):
     if hdf_file is not None:
         with h5py.File(hdf_file, "a") as hdf:
             if "configs" not in hdf.keys():
-                hdftools.setup_hdf(hdf, data.loc[0], attr)
+                hdftools.setup_hdf(hdf, data, attr)
                 configs.initialize_hdf(hdf)
             if "weights" not in hdf.keys():
                 hdf.create_dataset("weights", weights.shape)
-            for i in range(len(data)):
-                hdftools.append_hdf(hdf, data.loc[i])
+            hdftools.append_hdf(hdf, data)
             configs.to_hdf(hdf)
             hdf["weights"][:] = weights
 
@@ -260,6 +273,8 @@ def rundmc(
     propagate=dmc_propagate,
     feedback=1.0,
     hdf_file=None,
+    client=None,
+    npartitions=None,
     **kwargs,
 ):
     """
@@ -268,7 +283,7 @@ def rundmc(
     Args:
       wf: A Wave function-like class. recompute(), gradient(), and updateinternals() are used, as well as anything (such as laplacian() ) used by accumulators
 
-      configs: (nconfig, nelec, 3) - initial coordinates to start calculation.
+      configs: (nconfig, nelec, 3) - initial coordinates to start calculation. 
 
       weights: (nconfig,) - initial weights to start calculation, defaults to uniform.
 
@@ -296,68 +311,95 @@ def rundmc(
       weights: The final weights from this calculation
       
     """
-    # Restart
-    if hdf_file is not None:
-        with h5py.File(hdf_file, "a") as hdf:
-            if "configs" in hdf.keys():
-                stepoffset = hdf["step"][-1] + 1
-                configs.load_hdf(hdf)
-                weights = np.array(hdf["weights"])
-                if verbose:
-                    print("Restarted calculation")
+    # Restart from HDF file
+    if hdf_file is not None and os.path.isfile(hdf_file):
+        with h5py.File(hdf_file, "r") as hdf:
+            stepoffset = hdf["step"][-1] + 1
+            configs.load_hdf(hdf)
+            weights = np.array(hdf["weights"])
+            eref = hdf["eref"][-1]
+            esigma = hdf["esigma"][-1]
+            if verbose:
+                print("Restarted calculation")
+    else:
+        warmup = 2
+        df, configs = mc.vmc(
+            wf,
+            configs,
+            accumulators=accumulators,
+            client=client,
+            npartitions=npartitions,
+            verbose=verbose,
+        )
+        en = df[ekey[0] + ekey[1]][warmup:]
+        eref = np.mean(en).real
+        esigma = np.sqrt(np.var(en) * np.mean(df["nconfig"]))
+        if verbose:
+            print("eref start", eref, "esigma", esigma)
 
-    nconfig, nelec = configs.configs.shape[0:2]
+    nconfig = configs.configs.shape[0]
     if weights is None:
         weights = np.ones(nconfig)
 
     npropagate = int(np.ceil(nsteps / branchtime))
     df = []
-
-    df_, configs, weights = propagate(
-        wf,
-        configs,
-        weights,
-        tstep,
-        branchcut_start=1e8,
-        branchcut_stop=1e9,
-        eref=0.0,
-        nsteps=1,
-        stepoffset=0,
-        accumulators=accumulators,
-        ekey=ekey,
-        drift_limiter=drift_limiter,
-        **kwargs,
-    )
-
-    df_ = pd.DataFrame(df_)
-    eref = df_[ekey[0] + ekey[1]][0].real
-    esigma = np.abs(eref) / 100
     for step in range(npropagate):
-        if verbose:
-            print("-", end='', flush=True)
-        df_, configs, weights = propagate(
-            wf,
-            configs,
-            weights,
-            tstep,
-            branchcut_start * esigma,
-            branchcut_stop * esigma,
-            eref=eref,
-            nsteps=branchtime,
-            stepoffset=branchtime * step + stepoffset,
-            accumulators=accumulators,
-            ekey=ekey,
-            drift_limiter=drift_limiter,
-            **kwargs,
-        )
-        dmc_file(hdf_file, df_, dict(tstep=tstep), configs, weights)
+        if client is None:
+            df_, configs, weights = dmc_propagate(
+                wf,
+                configs,
+                weights,
+                tstep,
+                branchcut_start * esigma,
+                branchcut_stop * esigma,
+                eref=eref,
+                nsteps=branchtime,
+                accumulators=accumulators,
+                ekey=ekey,
+                drift_limiter=drift_limiter,
+                **kwargs,
+            )
+        else:
+            df_, configs, weights = dmc_propagate_parallel(
+                wf,
+                configs,
+                weights,
+                client,
+                npartitions,
+                tstep,
+                branchcut_start * esigma,
+                branchcut_stop * esigma,
+                eref=eref,
+                nsteps=branchtime,
+                accumulators=accumulators,
+                ekey=ekey,
+                drift_limiter=drift_limiter,
+                **kwargs,
+            )
+
         df_["eref"] = eref
+        df_["step"] = step + stepoffset
+        df_["esigma"] = esigma
+        df_["tstep"] = tstep
+        df_["weight_std"] = np.std(weights)
+        df_["nsteps"] = branchtime
+
+        dmc_file(hdf_file, df_, {}, configs, weights)
         # print(df_)
         df.append(df_)
-        eref = df_[ekey[0] + ekey[1]].values[-1].real - feedback * np.log(
-            np.mean(weights)
-        )
+        eref = df_[ekey[0] + ekey[1]] - feedback * np.log(np.mean(weights))
         configs, weights = branch(configs, weights)
-    if verbose: 
-        print("dmc done")
-    return pd.concat(df).reset_index(), configs, weights
+        if verbose:
+            print(
+                "energy",
+                df_[ekey[0] + ekey[1]],
+                "eref",
+                df_["eref"],
+                "sigma(w)",
+                df_["weight_std"],
+            )
+
+    df_ret = {}
+    for k in df[0].keys():
+        df_ret[k] = np.asarray([d[k] for d in df])
+    return df_ret, configs, weights
